import { InMemoryCache } from "apollo-cache-inmemory";
import { ApolloClient } from "apollo-client";
import { HttpLink } from "apollo-link-http";
import { setContext } from "apollo-link-context";
import { CredentialsService } from "../services/credentialsService";
import { Configuration } from "../config";

// tslint:disable-next-line: no-var-requires`
(global as any).fetch = require("node-fetch");

const apigeeHttpLink = new HttpLink({
  uri: Configuration().apigeeDBUrl,
});

const hasuraHttpLink = new HttpLink({
  uri: Configuration().hasuraDBUrl,
});

let authLinkAuthHeader = setContext((_, { headers }) => {
  // get the authentication token from local storage if it exists
  // return the headers to the context so httpLink can read them
  return {
    headers: {
      ...headers,
      authorization: CredentialsService.getAuthHeaderValue()
    },
  };
});

let authLinkApiKey = setContext((_, { headers }) => {
  // get the authentication token from local storage if it exists
  // return the headers to the context so httpLink can read them
  return {
    headers: {
      ...headers,
      "x-api-key": CredentialsService.getApiKey()
    },
  };
});

authLinkAuthHeader = authLinkAuthHeader.concat(hasuraHttpLink);

authLinkApiKey = authLinkApiKey.concat(apigeeHttpLink);

export const clientAuthHeader = new ApolloClient({
  link: authLinkAuthHeader,
  cache: new InMemoryCache({
    addTypename: false,
  }),
  defaultOptions: {
    query: {
      fetchPolicy: "network-only",
    },
    watchQuery: {
      fetchPolicy: "network-only",
    },
  },
});

export const clientApiKey = new ApolloClient({
  link: authLinkApiKey,
  cache: new InMemoryCache({
    addTypename: false,
  }),
  defaultOptions: {
    query: {
      fetchPolicy: "network-only",
    },
    watchQuery: {
      fetchPolicy: "network-only",
    },
  },
});

