import { gql } from "apollo-boost";

export const REQUEST_INFO = gql`
  query requestInfo($requestId: uuid!) {
    request(where: { id: { _eq: $requestId } }) {
      id
      type
      status
      rquestedBy {
        id
        user_name
        display_name
        email_id
      }
      approvedBy {
        id
        display_name
      }
      requested_data
      comments
    }
  }
`;

export const REQUESTED_APP = gql`
  query requestedApp($resourceId: uuid!) {
    app_product(where: { id: { _eq: $resourceId } }) {
      app {
        id
        display_name
        attributes
      }
      environment {
        name
      }
    }
  }
`;

export const ALL_APPROVERS = gql`
  query allApprovers($appId: uuid!) {
    app(where: { id: { _eq: $appId } }) {
      groups(where: { kind: { _eq: "approver" } }) {
        group {
          id
          users {
            user {
              user_name
              display_name
            }
          }
        }
      }
    }
  }
`;


export const ALL_DEVELOPERS_APP = gql`
  query allApprovers($appName: String!) {
    app(where: { name: { _eq: $appName } }) {
      id
      display_name
      attributes
      groups(where: { kind: { _eq: "developer" } }) {
        group {
          id
          users {
            user {
              user_name
            }
          }
        }
      }
    }
  }
`;

export const APPROVER = gql`
  query userInfo($approverId: uuid!) {
    user(where: { id: { _eq: $approverId } }) {
      user_name
      display_name
    }
  }
`;

export const SUBJECT_NAME = gql`
  query subjectName($subjectId: uuid!) {
    user(where: { id: { _eq: $subjectId } }) {
      display_name
      user_name
    }
  }
`;

export const PALADIN_ADMINS = gql`
  query PaladinAdmin {
    group(where: { name: { _eq: "paladin-admins" } }) {
      users {
        user {
          display_name
          user_name
        }
      }
    }
  }
`;

export const GET_APP_DETAILS_BY_NAME = gql`
  query getAppDetailsByName($appName: String) {
    app(where: { name: { _eq: $appName } }) {
      id
      name
      attributes
      tenant_id
    }
  }
`;

export const UPDATE_APP_ATTRIBUTES = gql`
mutation updateAppAttribute($appId: uuid!, $attributes: jsonb!) {
  update_app(where: {id: {_eq: $appId}}, _set: {attributes: $attributes}) {
    affected_rows
  }
}
`;

export const GET_ALL_SERVICE_APPS = gql`
  query getServiceApps {
    app(where: { attributes: { _contains: [{ name: "app-type", value: "service" }] } }) {
      id
      name
      display_name
      attributes
    }
  }
`;

export const GET_TENANT_DETAILS_BY_ID = gql`
  query getTenantInfo($tenantId: uuid!) {
    tenant(where: { id: { _eq: $tenantId } }) {
      name
      attributes
    }
  }
`;

export const UPDATE_REQUEST_COMMENTS = gql`
mutation RequestEdit($requestId: uuid!, $requestUpdateInput: jsonb!) {
  update_request(where: { id: { _eq: $requestId } }, _set: {comments: $requestUpdateInput}) {
    affected_rows
  }
}
`;