import { clientApiKey } from "../enviroments/local_environment";
import {
  REQUEST_INFO,
  ALL_APPROVERS,
  REQUESTED_APP,
  APPROVER,
  PALADIN_ADMINS,
  SUBJECT_NAME,
  ALL_DEVELOPERS_APP,
  GET_APP_DETAILS_BY_NAME,
  UPDATE_APP_ATTRIBUTES,
  GET_TENANT_DETAILS_BY_ID,
} from "../graphqlQueries/request-info.query";
import _ from "lodash";
import { Logger, LambdaLogger } from "../lib/lambda-logger/index";
import { NitinolService } from "./nitinolService";
import { ApolloQueryResult } from "apollo-boost";

export class AuthNotificationService {
  public log: Logger = LambdaLogger.getInstance("AuthNotificationService");
  staus = ["new","inprogress","pending"];
  public async sendSecretManagementNotification(
    appName: string,
    remainingDays: string,
    resetSecret: boolean,
    resetSecretStatusCode: number,
    isManualReset: boolean
  ) {
    let autoSecretRotation;

    const appInfo = await this.getDataFromQuery(ALL_DEVELOPERS_APP, {
      appName: appName,
    });

    const tenantId = await this.getTenantIdByAppName(appName);
    const tenantAttributes = await this.getTenantAttributesByTenantId(tenantId);
    const tenantTemplateName = tenantAttributes['tenant_template_name'];
    const tenantEmailFromAddress = tenantAttributes['tenant_email_fromaddress'];

    let appDevelopers: any = _.map(
      appInfo.app[0].groups[0].group.users,
      "user.user_name"
    );

    if (_.isEmpty(appDevelopers)) {
      this.log.info(
        "Unable to fetch Developers List of Email Addresses for an App from DB"
      );
    }

    var paladinPortalUrl = "paladin-portal.corp.hpicloud.net/";

    if (process.env.stage != "prod") {
      var envStage = "dev";
      if (process.env.stage === "test") {
        envStage = "stg";
      }

      paladinPortalUrl =
        "https://paladin-portal-" + envStage + ".corp.hpicloud.net/";
    }
    paladinPortalUrl += "apps/" + appInfo.app[0].id;

    let auto = (!isManualReset)?' auto':'';
    let action = (!isManualReset)?' No action required from your end.':'';
    const templateData = {
      remainingDays: remainingDays,
      env: process.env.stage,
      appName: appInfo.app[0].display_name,
      paladinPortalUrl: paladinPortalUrl,
      tenantName: tenantTemplateName,
      tenantEmailFromAddress: tenantEmailFromAddress,
      auto: auto,
      action: action
    };

    if (appInfo.app[0].attributes && appInfo.app[0].attributes.length > 0) {
      autoSecretRotation = _.find(appInfo.app[0].attributes, { name: 'enable-auto-secret-rotation', value: 'true' });
    }
    if (autoSecretRotation) {
      if (resetSecret) {
        if (resetSecretStatusCode === 200) {
          try {
            if (process.env.stage == "prod") {
              const serviceAppDetails = await this.getDataFromQuery(GET_APP_DETAILS_BY_NAME, {
                appName: appName,
              });
              if (serviceAppDetails.app && serviceAppDetails.app.length > 0) {
                let appId = serviceAppDetails.app[0].id;
                let attributes = serviceAppDetails.app[0].attributes;
                const appType = attributes.find(({ name }) => name === 'app-type').value;
                attributes = _.filter(attributes, function (p) {
                  return p.name !== 'secret-created-date' && p.name !== 'secret-expiry-date';
                });
                if (appType === 'service') {
                  let secretCreatedDate = [{ name: 'secret-created-date', value: Math.round(Date.now() / 1000) }];
                  attributes = [...attributes, ...secretCreatedDate];
                  let updated: any = await this.updateAppAttributes(appId, attributes);
                  if (updated) {
                    this.log.info("Updated the app attributes Value");
                  }
                }
              }
            }
          }
          catch (error) {
            this.log.info("Failed to get app data");
          }
          var params = {
            templateName: "secretExpiryAutoRotationSuccessEmail",
            toAddress: appDevelopers,
            subject: tenantTemplateName + " Auth - " +
              _.upperCase(process.env.stage) +
              " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
            templateData: templateData,
          };

        } else {
          var params = {
            templateName: "secretExpiryAutoRotationFailedEmail",
            toAddress: appDevelopers,
            subject: tenantTemplateName + " Auth - " +
              _.upperCase(process.env.stage) +
              " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
            templateData: templateData,
          };
        }

      } else {
        if (remainingDays && parseInt(remainingDays) === 15 || parseInt(remainingDays) === 8) {
          var params = {
            templateName: "secretExpiryInformationAutoEmail",
            toAddress: appDevelopers,
            subject: tenantTemplateName + " Auth - " +
              _.upperCase(process.env.stage) +
              " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
            templateData: templateData,
          };
      } else {
          return {  status: 'Auto-Rotation info email should be sent on 15th and 8th day' };
        }
      }
    }
    else {
      if (resetSecret) {
        if (resetSecretStatusCode === 200) {
          var params = {
            templateName: "secretExpiryAutoRotationSuccessEmail",
            toAddress: appDevelopers,
            subject: tenantTemplateName + " Auth - " +
              _.upperCase(process.env.stage) +
              " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
            templateData: templateData,
          };

        } else {
          var params = {
            templateName: "secretExpiryAutoRotationFailedEmail",
            toAddress: appDevelopers,
            subject: tenantTemplateName + " Auth - " +
              _.upperCase(process.env.stage) +
              " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
            templateData: templateData,
          };
        }
      } else {
        var params = {
          templateName: "secretExpiryWarningEmail",
          toAddress: appDevelopers,
          subject: tenantTemplateName + " Auth - " +
            _.upperCase(process.env.stage) +
            " : Secret Expiry Warning for " + appInfo.app[0].display_name + ".",
          templateData: templateData,
        };
      }
    }
    params["fromAddress"] = tenantEmailFromAddress;
    this.log.info("Sending email to: " + appDevelopers);
    var notificationStatus = await this.sendEmail(params);
    return notificationStatus;
  }

  public async sendNotification(requestMessage: any) {
    const requestId = _.get(requestMessage, "event.data.new.id");
    const operation = _.get(requestMessage, "event.op");

    if (!requestId) {
      this.log.error("Request id is not present in the request message.");
      return;
    }
    const requestInfo = await this.getDataFromQuery(REQUEST_INFO, {
      requestId,
    });

    const resourceId = _.get(requestInfo, "request[0].requested_data.resource");
    let admins: any;
    let approvers: any;
    let requestedApp: any;
    let sendUsingCustomEmailIdFlag: boolean = false;
    let customEmailId: any;

    if (!resourceId) {
      const paladinAdmin = await this.getDataFromQuery(PALADIN_ADMINS, {});
      admins = _.map(paladinAdmin.group[0].users, "user.user_name");
      this.log.info(
        "no resource for this request, hence sending the email to paladin admins",
        admins
      );
    } else {
      requestedApp = await this.getDataFromQuery(REQUESTED_APP, {
        resourceId,
      });

      const isCustomEmailId = _.find(requestedApp.app_product[0].app.attributes, { name: "notification-emailId" });

      if (isCustomEmailId) {
        sendUsingCustomEmailIdFlag = true;
        customEmailId = isCustomEmailId.value;
      }
      else {
        sendUsingCustomEmailIdFlag = false;
      }

      this.log.info("Customize email : ", sendUsingCustomEmailIdFlag, " : ", customEmailId);

      const appId = _.get(requestedApp, "app_product[0].app.id");
      if (!appId) {
        this.log.error("Unable to fetch App id from GraphQL Api");
        return;
      }

      const allApprovers = await this.getDataFromQuery(ALL_APPROVERS, {
        appId: requestedApp.app_product[0].app.id,
      });
      approvers = _.map(
        allApprovers.app[0].groups[0].group.users,
        "user.user_name"
      );

      if (_.isEmpty(approvers)) {
        this.log.info(
          "Unable to fetch Approvers List of Email Addresses for an App from DB"
        );
      }
    }

    const params: any = await this.getDataFromPayload(
      requestMessage,
      requestInfo,
      requestedApp,
      approvers,
      admins
    );

    if (!params) {
      this.log.info("No Data to send email");
      return;
    }

    if (sendUsingCustomEmailIdFlag) {
      params["fromAddress"] = customEmailId;
    }
    else if(params.templateData.tenantEmailFromAddress){
      params["fromAddress"] = params.templateData.tenantEmailFromAddress;
    }
    else {
      params["fromAddress"] = null;
    }

    if (!_.isEmpty(params.toAddress)) {
      await this.sendEmail(params);
    }

    if (params.templateName === "emailToAllApprovers") {
      if (operation === "INSERT") {
        params.templateName = "emailToRequester";
        params.subject =
        params.templateData.tenantName +" Auth - " +
          _.upperCase(process.env.stage) +
          " : Your request is submitted successfully.";
      }
      if (operation === "UPDATE") {
        params.templateName = "requestSentBackEmail";
        params.subject =
        params.templateData.tenantName +" Auth - " +
          _.upperCase(process.env.stage) +
          " : Your request is Sentback.";
      }
      params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
      this.log.info("Sending email to requester");
      await this.sendEmail(params);
    }
    if (params.templateName === "reactivateEmailToAllApprovers") {
      if (operation === "INSERT") {
        params.templateName = "reactivationEmailToRequester";
        params.subject =
        params.templateData.tenantName +" Auth - " +
          _.upperCase(process.env.stage) +
          " : Your request is submitted successfully.";
      }
      if (operation === "UPDATE") {
        params.templateName = "requestSentBackEmail";
        params.subject =
        params.templateData.tenantName +" Auth - " +
          _.upperCase(process.env.stage) +
          " : Your request is Sentback.";
      }
      params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
      this.log.info("Sending email to requester");
      await this.sendEmail(params);
    }
    if (params.templateName === "reactivationEmailAppLevelToApprover") {
      if (operation === "INSERT") {
        params.templateName = "reactivationEmailAppLevelToRequester";
        params.subject =
        params.templateData.tenantName +" Auth - " +
          _.upperCase(process.env.stage) +
          " : Your app level reactivation request is submitted successfully.";
      }
      if (operation === "UPDATE") {
        params.templateName = "requestSentBackEmail";
        params.subject =
        params.templateData.tenantName +" Auth - " +
          _.upperCase(process.env.stage) +
          " : Your app level reactivation request is Sentback.";
      }
      params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
      this.log.info("Sending email to requester");
      await this.sendEmail(params);
    }
    if (params.templateName === "emailToAllApproversAfterResubmit") {
      params.templateName = "emailToAllRequesterAfterResubmit";
      params.subject =
      params.templateData.tenantName +" Auth - " +
        _.upperCase(process.env.stage) +
        " : Your request is re-submitted successfully.";
      params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
      this.log.info("Sending email to requester");
      await this.sendEmail(params);
    }
    if (params.templateName === "reactivateEmailToAllApproversAfterResubmit") {
      params.templateName = "reactivateEmailToAllRequesterAfterResubmit";
      params.subject =
      params.templateData.tenantName +" Auth - " +
        _.upperCase(process.env.stage) +
        " : Your request is re-submitted successfully.";
      params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
      this.log.info("Sending email to requester");
      await this.sendEmail(params);
    }
    if (params.templateName === "reactivationEmailAppLevelToAllApproverAfterResubmit") {
      params.templateName = "reactivationEmailAppLevelToAllRequesterAfterResubmit";
      params.subject =
      params.templateData.tenantName +" Auth - " +
        _.upperCase(process.env.stage) +
        " : Your app level reactivation request is re-submitted successfully.";
      params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
      this.log.info("Sending email to requester");
      await this.sendEmail(params);
    }
    if (params.templateName === "emailToAllApproversAfterAccept") {
      let approvedBy = requestInfo.request[0].approvedBy ? requestInfo.request[0].approvedBy.display_name : "";
      params.templateName = "emailToAllRequesterAfterAccept";
      params.subject =
      params.templateData.tenantName +" Auth - " +
        _.upperCase(process.env.stage) +
        " : Your request is accepted by " + approvedBy;
      params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
      this.log.info("Sending email to requester");
      await this.sendEmail(params);
    }
  }

  private async sendEmail(params: any) {
    this.log.info("Calling nitinol service to send email with params", params);
    const notificationRes = await NitinolService.sendEmail(
      params.templateName,
      params.toAddress,
      params.fromAddress,
      null,
      null,
      params.subject,
      params.templateData
    );
    return notificationRes;
  }

  private async updateAppAttributes(
    appId: string,
    attributes: string
  ) {
    let result: any = {};
    try {
      result = await clientApiKey.mutate({
        mutation: UPDATE_APP_ATTRIBUTES,
        variables: {
          appId: appId,
          attributes: attributes
        },
      });
      this.log.info("Updated the secret created date in app attributes table: ", result);
      return result;
    } catch (err) {
      this.log.error("Error while  updating app attributes data in DB", err);
      return;
    }
  }

  private async getDataFromQuery(
    query: any,
    variables: any
  ): Promise<any | undefined> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query,
        variables,
      });
      this.log.info(
        "Received DB Call Result: ",
        result && result.data ? JSON.stringify(result.data) : "NULL"
      );
      return result.data;
    } catch (err) {
      this.log.error("Error while querying data from DB", err);
      return;
    }
  }

  private async getDataFromPayload(
    requestMessage: any,
    requestInfo: any,
    requestedApp: any,
    approversArr: any,
    admins: any
  ): Promise<any | undefined> {
    let params = {};
    let environmentRequestUrl: any;
    try {
      const requestStatus = _.get(requestMessage, "event.data.new.status");
      const tenantId = _.get(requestMessage, "event.data.new.tenant_id");
      const requestId = _.get(requestMessage, "event.data.new.id");
      const comments = requestInfo.request[0].comments;
      const latestComment = comments ? comments[comments.length - 1] : "";
      const subjectId = requestInfo.request[0].requested_data.subject;
      const requestCount = _.get(requestMessage, "event.data.new.notification_data.count");
      const requestCountOld = _.get(requestMessage, "event.data.old.notification_data.count");
      const subjectName = await this.getDataFromQuery(SUBJECT_NAME, {
        subjectId,
      });
      const tenantAttributes = await this.getTenantAttributesByTenantId(tenantId);
      const tenantTemplateName = tenantAttributes['tenant_template_name'];
      const tenantEmailFromAddress = tenantAttributes['tenant_email_fromaddress'];
      const regExpForURLs =
      /((http|https):\/\/)?(ftp:\/\/)?(www\.)?(http:\/\/(.*?)\/)?[a-zA-Z0-9@:%._\\+~#?&//=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%._\\+~#?&//=]*)/g;

      let customEmailId;
      var environmentUrl = "paladin-portal.corp.hpicloud.net/home";
      if (process.env.stage != "prod") {
        var envStage = "dev";
        if (process.env.stage === "test") {
          envStage = "stg";
        }
        environmentUrl =
          "https://paladin-portal-" + envStage + ".corp.hpicloud.net/home";
      }
      var callbackUrl = _.find(requestedApp?.app_product[0].app.attributes, { name: "callback-url" });
      if (callbackUrl) {
        environmentRequestUrl = callbackUrl.value;
      }
      else {
        environmentRequestUrl = '';
      }
      if (requestedApp && requestedApp.app_product && requestedApp.app_product.length > 0) {
        customEmailId = _.find(requestedApp.app_product[0].app.attributes, { name: "notification-emailId" });
      }
      var dontReply = "";
      if (customEmailId) {
        dontReply = "";
      }
      else {
        dontReply = "Please do not reply to this email. We are unable to respond to inquiries sent to this address.";
      }
      const templateData = {
        environmentUrl: environmentUrl,
        environmentRequestUrl,
        dontReply,
        requestId,
        username: requestInfo.request[0].rquestedBy.display_name,
        type: _.upperCase(requestInfo.request[0].type),
        status: _.upperCase(requestInfo.request[0].status),
        app: requestedApp ? requestedApp.app_product[0].app.display_name : "",
        subjectName: subjectName.user[0].display_name,
        subjectEmailAdd: subjectName.user[0].user_name,
        enviroment: requestedApp
          ? _.upperCase(requestedApp.app_product[0].environment.name)
          : _.upperCase(process.env.stage),
        environmentReactivate: _.upperCase(process.env.stage),
        requestedBy: requestInfo.request[0].rquestedBy.display_name,
        whatAccess: (requestInfo.request[0].requested_data.whatAccess && requestInfo.request[0].requested_data.whatAccess.info) ? requestInfo.request[0].requested_data.whatAccess.info : requestInfo.request[0].requested_data.whatAccess.length > 100 ? 
        requestInfo.request[0].requested_data.whatAccess.replace(regExpForURLs, "").substring(0,100)+'...' :
        requestInfo.request[0].requested_data.whatAccess.replace(regExpForURLs, ""),
        whyAccess: requestInfo.request[0].requested_data.whyAccess.length > 100 ? 
        requestInfo.request[0].requested_data.whyAccess.replace(regExpForURLs, "").substring(0,100)+'...' : 
        requestInfo.request[0].requested_data.whyAccess.replace(regExpForURLs, ""),
        email_id: requestInfo.request[0].rquestedBy.email_id,
        comment: latestComment.note ? latestComment.note : "",
        approverUsername: requestInfo.request[0].approvedBy
          ? requestInfo.request[0].approvedBy.display_name
          : "",
        tenantName: tenantTemplateName,
        tenantEmailFromAddress: tenantEmailFromAddress
      };

      /*
        When request is new or pending or inprogress:
        1. Cron job will run to send the reminder for auto reject
      */  
      if (this.staus.includes(requestStatus) && requestCount && (requestCount != requestCountOld)) { //&& requestCount != 0
        if (requestCount <= 3) {
          params = {
            templateName: "requestWaitingEmail",
            toAddress: _.isEmpty(approversArr) ? admins : approversArr,
            subject:
              "Reminder-" + requestCount + " - "+ tenantTemplateName +" Auth - " +
              _.upperCase(process.env.stage) +
              " : The request is waiting for your action ",
            templateData,
          };
        }
        this.log.info("params", params);
      }

      /* 
        Request status is new :
        1. New request raised
        2. Request resubmitted after it is sentback by approver
      */
      if (
        requestStatus === "new" &&
        requestInfo.request[0].type !== "reactivation" && (!requestCount || requestCount == requestCountOld)//|| requestCount == 0
      ) {
        const oldStatus = _.get(requestMessage, "event.data.old.status");

        // incase when request is sentback
        if (oldStatus === "inprogress") {
          params = {
            templateName: "emailToAllApprovers",
            templateData,
          };
          return params;
        }
        let subject = (oldStatus) ? ' : A request has been re-submitted and waiting for approval.' : ' : A new request has been submitted and waiting for approval.'
        params = {
          templateName: oldStatus
            ? "emailToAllApproversAfterResubmit"
            : "emailToAllApprovers",
          toAddress: _.isEmpty(approversArr) ? admins : approversArr,
          subject:
          tenantTemplateName +" Auth - " +
            _.upperCase(process.env.stage) +
            subject,
          templateData,
        };
      }

      // when request is accepted
      if (requestStatus === "inprogress" && (!requestCount || requestCount == requestCountOld)) {
        params = {
          templateName: "emailToAllApproversAfterAccept",
          toAddress: _.isEmpty(approversArr) ? admins : approversArr,
          subject:
           tenantTemplateName +" Auth - " +
            _.upperCase(process.env.stage) +
            " : The request is accepted by " + templateData.approverUsername,
          templateData,
        };
        this.log.info("params", params);
      }

      if (requestInfo.request[0].type === "reactivation") {

        if (requestInfo.request[0].requested_data.resource) {
          //when user is disabled at app level, and raises an account reactivation request
          templateData.type = templateData.type + " - App Level";
          if (requestStatus === "new") {
            const oldStatus = _.get(requestMessage, "event.data.old.status");
            // incase when request is sentback
            if (oldStatus === "inprogress") {
              params = {
                templateName: "reactivationEmailAppLevelToApprover",
                templateData,
              };
              return params;
            }
            let subject = (oldStatus) ? ' : App Level reactivation request is re-submitted.' : ' : App Level reactivation request is raised.'
            params = {
              templateName: oldStatus
                ? "reactivationEmailAppLevelToAllApproverAfterResubmit"
                : "reactivationEmailAppLevelToApprover",
              toAddress: _.isEmpty(approversArr) ? admins : approversArr,
              subject:
              tenantTemplateName +" Auth - " +
                _.upperCase(process.env.stage) +
                subject,
              templateData,
            };
          }

          this.log.info("params", params);
        }
        else {
          // when user is disabled, and raises an account reactivation request
          if (requestStatus === "new") {
            const oldStatus = _.get(requestMessage, "event.data.old.status");

            // incase when request is sentback
            if (oldStatus === "inprogress") {
              params = {
                templateName: "reactivateEmailToAllApprovers",
                templateData,
              };
              return params;
            }
            let subject = (oldStatus) ? ' : Account reactivation request is re-submitted.' : ' : Account reactivation request is raised.'
            params = {
              templateName: oldStatus
                ? "reactivateEmailToAllApproversAfterResubmit"
                : "reactivateEmailToAllApprovers",
              toAddress: _.isEmpty(approversArr) ? admins : approversArr,
              subject:
              tenantTemplateName +" Auth - " +
                _.upperCase(process.env.stage) +
                subject,
              templateData,
            };
          }
          this.log.info("params", params);
        }
      }

      // when request is reassigned
      if (requestStatus === "pending" && (!requestCount || requestCount == requestCountOld)) {
        const approverId = _.get(requestMessage, "event.data.new.approved_by");
        if (!approverId) {
          this.log.error(
            "Request hasn't been assigned to any approver, hence unable to get approver id"
          );
          return;
        }
        if (approverId) {
          const approverEmailAdd = await this.getDataFromQuery(APPROVER, {
            approverId,
          });

          if (!approverEmailAdd) {
            this.log.error(
              "Unable to fetch Approver Email Address for an App from DB, hence not sending the email"
            );
            return;
          }
          params = {
            templateName: "reassignRequestEmail",
            toAddress: [approverEmailAdd.user[0].user_name],
            subject:
            tenantTemplateName +" Auth - " +
              _.upperCase(process.env.stage) +
              " : A new request has been assigned to you, waiting for approval.",
            templateData,
          };
        }
      }

      // when request is approved
      if (requestStatus === "approved") {
        params = {
          templateName: "requestApprovedEmail",
          toAddress: [requestInfo.request[0].rquestedBy.user_name],
          subject:
          tenantTemplateName +" Auth - " +
            _.upperCase(process.env.stage) +
            " : Your request is Approved successfully.",
          templateData,
        };
      }

      // when request is rejected
      if (requestStatus === "rejected") {
        params = {
          templateName: "requestRejectedEmail",
          toAddress: [requestInfo.request[0].rquestedBy.user_name],
          subject:
          tenantTemplateName +" Auth - " +
            _.upperCase(process.env.stage) +
            " : Your request is Rejected.",
          templateData,
        };
        if (requestCount && requestCount == 4) {
          params = {
            templateName: "requestAutoRejectedEmail",
            toAddress: [requestInfo.request[0].rquestedBy.user_name],
            subject:
            tenantTemplateName +" Auth - " +
              _.upperCase(process.env.stage) +
              " : Your request is Auto Rejected.",
            templateData,
          };
        }
      }
      return params;
    } catch (err) {
      this.log.error("Error while building email params:", err);
      return;
    }
  }
  
  public async sendInactiveUserNotification(
    userEmail: any,
    emailContent: any,
    tenant_id: any
  ) {
    const tenantAttributes = await this.getTenantAttributesByTenantId(
      tenant_id
    );
    const tenantTemplateName = tenantAttributes["tenant_template_name"];
    const tenantEmailFromAddress = tenantAttributes["tenant_email_fromaddress"];
    const templateData = {
      emailContent: emailContent,
      env: process.env.stage,
      dontReply:
        "Please do not reply to this email. We are unable to respond to inquiries sent to this address.",
      tenantName: tenantTemplateName,
    };

    userEmail = userEmail.split(" ");
    const params = {
      templateName: "paladinUserInactiveNotification",
      toAddress: userEmail,
      fromAddress: tenantEmailFromAddress,
      subject:
        tenantTemplateName +
        " Auth - " +
        _.upperCase(process.env.stage) +
        " : Account Deactivation Reminder.",
      templateData: templateData,
    };
    this.log.info("params:", params);
    try {
      this.log.info("Sending email to: " + userEmail);
      const notificationStatus = await this.sendEmail(params);
      return notificationStatus;
    } catch (err) {
      this.log.error("Sending email failed", err);
    }
  }

  public async sendPortalNotification(
    usersEmailList: any,
    emailContent: string,
    tenantId: string,
  ) {

    const tenantAttributes = await this.getTenantAttributesByTenantId(tenantId);
    const tenantTemplateName = tenantAttributes['tenant_template_name'];
    const tenantEmailFromAddress = tenantAttributes['tenant_email_fromaddress'];
    const templateData = {
      emailContent: emailContent,
      env: process.env.stage,
      dontReply: "Please do not reply to this email. We are unable to respond to inquiries sent to this address.",
      tenantName: tenantTemplateName
    };

    var params = {
      templateName: "portalNotificationEmail",
      toAddress: usersEmailList,
      fromAddress: tenantEmailFromAddress,
      subject: tenantTemplateName + " Auth - " +
        _.upperCase(process.env.stage) +
        " : Paladin Platform Notification.",
      templateData: templateData,
    };

    try {
      if (usersEmailList.length > 45) {
        var chunkSize = 45;
        for (let i = 0; i <= usersEmailList.length; i += chunkSize) {
          const chunkEmailIds = usersEmailList.slice(i, i + chunkSize);
          params.toAddress = chunkEmailIds;
          this.log.info("Sending batch email: " + chunkEmailIds);
          var notificationStatus = await this.sendEmail(params);
        }
      }
      else {
        this.log.info("Sending email to : " + usersEmailList);
        var notificationStatus = await this.sendEmail(params);
      }
      return notificationStatus;
    } catch (err) {
      this.log.error("Sending email failed", err);
    }
  }

  public async getTenantAttributesByTenantId(tenantId: any) {
    const tenantDetails = await this.getDataFromQuery(GET_TENANT_DETAILS_BY_ID, {
      tenantId
    });
    let tenant_template_name: any;
    let tenantEmailFromAddress:any = null;

    if (tenantDetails.tenant && tenantDetails.tenant[0].attributes.length > 0) {
      for await (const attribute of tenantDetails.tenant[0].attributes) {
        if (attribute.name == 'tenant-template-name') {
          tenant_template_name = attribute.value
        }
        if (attribute.name == 'tenant-notification-fromaddress') {
          tenantEmailFromAddress = attribute.value
        }
      }
    }
      return {"tenant_template_name":tenant_template_name, "tenant_email_fromaddress":tenantEmailFromAddress};
  }

  public async getTenantIdByAppName(appName: String) {
    const getappTenantId = await this.getDataFromQuery(GET_APP_DETAILS_BY_NAME, {
      appName
    });
    if (getappTenantId.app && getappTenantId.app.length > 0) {
      return getappTenantId.app[0].tenant_id;
    }
    return;
  }

  public async sendJiraNotification(
    data,
    jiraDetails, tenantId
  ) {
    const tenantAttributes = await this.getTenantAttributesByTenantId(tenantId);
    const tenantTemplateName = tenantAttributes['tenant_template_name'];
    const tenantEmailFromAddress = tenantAttributes['tenant_email_fromaddress'];
    let emailContent = jiraDetails.fields.description;
    let reporter = jiraDetails.fields.reporter.name;
    let assignee = jiraDetails.fields.assignee.name;
    const usersEmailList = [reporter, assignee, "hp.mps.commonauth@groups.hp.com"];
    const templateData = {
      emailContent: emailContent,
      env: process.env.stage,
      dontReply: "Please do not reply to this email. We are unable to respond to inquiries sent to this address.",
      tenantName: tenantTemplateName
    };
  
    var params = {
      templateName: "paladinJiraNotification",
      toAddress: usersEmailList,
      fromAddress: tenantEmailFromAddress,
      subject: data.key,
      templateData: templateData,
    };
    this.log.info("params : " + params);
    try {      
      this.log.info("Sending email to : " + usersEmailList);
      let notificationStatus = await this.sendEmail(params);
      return notificationStatus;
    } catch (err) {
      this.log.error("Sending email failed", err);
    }
  }
}
