import { clientAuthHeader,clientApiKey } from "../enviroments/local_environment";
import {
  USER_POLICY_EXISTS,
  GET_USER_IDS,
  POLICY_APP_RESOURCE_EXISTS,
  GET_USER_POLICY_TAXONOMY,
  GET_APP_PRODUCT_ID,
  GET_PRODUCT_TAXONOMY_ID,
  CREATE_USER_POLICY,
  ADD_POLICY_RESOURCE,
  ADD_POLICY_TAXONOMY,
  UPDATE_POLICY_TAXONOMY,
  GET_APP_TAXON_USERS,
  GET_APP_ALL_USERS,
  CHECK_USER,
  ADD_USER,
  ADD_USER_TENANT_ASSOCIATION,
  GET_USER_AND_GROUP_ID,
  ADD_USER_TO_GROUP,
  GET_CREATED_BY_USER_ID,
  GET_USER_GROUPS,
  GET_DC_APP_ALL_USERS,
  REMOVE_USER_FROM_GROUP,
  DELETE_POLICY_RESOURCE,
  GET_EXTENDED_USERS,
  GET_APP_SUBJECT_NAME_DOCUMENT,
  UPDATE_REQUESTED_VALUE_MUTATION,
  CHECK_GROUPS_EXISTS,
  FETCH_APP_GROUPS,
  FETCH_APP_POLICY_RESOURCE,
  GET_POLICY_DETAILS,
  GET_POLICY_RESOURCE,
  GET_USER_EMAIL,
  CREATE_AUDIT_LOG,
  GET_APP_DETAILS,
  GET_PRODUCT_DETAILS,
  GET_PRODUCT_ID_BY_PERMISSION_ID,
  GET_ENTITY_NAME,
  GET_ENTITY_NAME_BY_POLICY_SUBJECT,
  GET_REQUEST,
  REQUEST_NOTIFICATION,
  UPDATE_REQUEST,
  GET_APP_ATTRIBUTES_BY_APP_NAME,
  SERVICE_DISCOVERY_DOCUMENT,
  GET_USER_POLICY_ALL_TAXONOMY,
  GET_PRODUCT_ALL_TAXONOMY,
  DISABLE_POLICY_RESOURCE,
  ENABLE_POLICY_RESOURCE,
  FETCH_ALL_APP_GROUPS,
  GET_GROUP_DETAILS,
  GET_PERSONA_DETAILS,
  GET_USER_DETAILS,
  UPDATE_REQUEST_COMMENTS,
  GET_All_USERS,
  GET_APP_LIST_FOR_USERS
} from "../graphqlQueries/extension-queries";
import { Logger, LambdaLogger } from "../lib/lambda-logger/index";
import { ApolloQueryResult } from "apollo-boost";
import { isArray } from "lodash";

export class ExtensionService {
  public log: Logger = LambdaLogger.getInstance("ExtensionService");

  public async userPolicyExists(emailId: string) {
    let policyInfo = {};
    try {
      policyInfo = await this.getDataFromQuery(USER_POLICY_EXISTS, {
        emailId,
      });
      this.log.info("UserPolicy Exists: ", policyInfo);
      return policyInfo;
    } catch (err) {
      this.log.error("Error while checking userTaxonExists data in DB", err);
      return;
    }
  }

  public async policyAppResourceExists(
    policyId: string,
    appName: string,
    productName: string
  ) {
    let policyResourceInfo = {};
    try {
      policyResourceInfo = await this.getDataFromQuery(
        POLICY_APP_RESOURCE_EXISTS,
        {
          policyId,
          appName,
          productName,
        }
      );
      this.log.info("PolicyAppResource Exists: ", policyResourceInfo);
      return policyResourceInfo;
    } catch (err) {
      this.log.error("Error while checking userTaxonExists data in DB", err);
      return;
    }
  }

  public async getUserPolicyTaxonomy(
    emailId: string,
    appName: string,
    productName: string
  ) {
    let taxonInfo: any = {};
    let taxonomyName: string = "customer-location";
    try {
      taxonInfo = await this.getDataFromQuery(GET_USER_POLICY_TAXONOMY, {
        emailId,
        appName,
        productName,
        taxonomyName,
      });
      this.log.info("UserTaxon exists: ", taxonInfo);
      return taxonInfo;
    } catch (err) {
      this.log.error("Error while checking userTaxonExists data in DB", err);
      return;
    }
  }

  public async getUserPolicyAllTaxonomy(
    emailId: string,
    appName: string,
    productName: string
  ) {
    let taxonInfo: any = {};
    try {
      taxonInfo = await this.getDataFromQuery(GET_USER_POLICY_ALL_TAXONOMY, {
        emailId,
        appName,
        productName,
      });
      this.log.info("UserPolicyAllTaxonomy exists: ", taxonInfo);
      return taxonInfo;
    } catch (err) {
      this.log.error("Error while checking userAllTaxonExists data in DB", err);
      return;
    }
  }

  public async getProductAllTaxonomy(
    appProductId: string
  ) {
    let taxonInfo: any = {};
    try {
      taxonInfo = await this.getDataFromQuery(GET_PRODUCT_ALL_TAXONOMY, {
        appProductId
      });
      this.log.info("getProductAllTaxonomy exists: ", taxonInfo);
      return taxonInfo;
    } catch (err) {
      this.log.error("Error while checking getProductAllTaxonomy data in DB", err);
      return;
    }
  }

  public async createUserPolicy(
    emailId: string,
    appName: string,
    productName: string,
    customerId: Number
  ) {
    const emailIds: string[] = ["chandru.k-s@hp.com", emailId];
    const userData = await this.getDataFromQuery(GET_USER_IDS, {
      emailIds,
    });
    let creatorId = "";
    let subjectId = "";
    userData.user.forEach((user) => {
      if (user.email_id === "chandru.k-s@hp.com") {
        creatorId = user.id;
        subjectId = user.id;
      } else if (user.email_id === emailId) {
        subjectId = user.id;
      }
    });

    const appProductData = await this.getDataFromQuery(GET_APP_PRODUCT_ID, {
      appName,
      productName,
    });

    const productTaxonomyNames = ["customer-location"];

    const productTaxonomyData = await this.getDataFromQuery(
      GET_PRODUCT_TAXONOMY_ID,
      {
        productName,
        productTaxonomyNames,
      }
    );

    var taxonValue = [{
      "customers": [customerId],
      "location": null
    }];

    var policyInsertInput = {
      created_by: creatorId,
      approved_by: creatorId,
      is_active: true,
      kind: "user",
      notes: "",
      name: null,
      statements: {
        data: {
          app_product_id: appProductData.app_product[0].id,
          taxon_allow: {
            data: [{
              product_taxonomy_id: productTaxonomyData.product_taxonomy[0].id,
              value: taxonValue,
            }],
          },
        },
      },
      subjects: {
        data: {
          subject: subjectId,
        },
      },
    };

    let result: any = {};
    try {
      result = await clientAuthHeader.mutate({
        mutation: CREATE_USER_POLICY,
        variables: {
          policyInsertInput,
        },
      });
      this.log.info("UserPolicy Created: ", result);
      return result;
    } catch (err) {
      this.log.error("Error while creating UserPolicy data in DB", err);
      return;
    }
  }

  public async AddPolicyResource(
    policyId: string,
    appName: string,
    productName: string,
    customerId: Number
  ) {
    const appProductData = await this.getDataFromQuery(GET_APP_PRODUCT_ID, {
      appName,
      productName,
    });

    const productTaxonomyNames = ["customer-location"];

    const productTaxonomyData = await this.getDataFromQuery(
      GET_PRODUCT_TAXONOMY_ID,
      {
        productName,
        productTaxonomyNames,
      }
    );

    var taxonValue = [{
      "customers": [customerId],
      "location": null
    }];

    var policyResourceInsertInput = {
      policy_id: policyId,
      app_product_id: appProductData.app_product[0].id,
      taxon_allow: {
        data: [{
          product_taxonomy_id: productTaxonomyData.product_taxonomy[0].id,
          value: taxonValue,
        }],
      },
    };

    let result: any = {};
    try {
      result = await clientAuthHeader.mutate({
        mutation: ADD_POLICY_RESOURCE,
        variables: {
          policyResourceInsertInput,
        },
      });
      this.log.info("Added PolicyResource: ", result);
      return result;
    } catch (err) {
      this.log.error("Error while Add PolicyResource data in DB", err);
      return;
    }
  }

  public async AddPolicyTaxonomy(
    policyResourceId: string,
    productName: string,
    value: string
  ) {
    const productTaxonomyNames = ["customer-location"];
    const productTaxonomyData = await this.getDataFromQuery(
      GET_PRODUCT_TAXONOMY_ID,
      {
        productName,
        productTaxonomyNames,
      }
    );

    var policyResourceTaxonAllowInsertInput: any = {
      policy_resource_id: policyResourceId,
      product_taxonomy_id: productTaxonomyData.product_taxonomy[0].id,
      value: value,
    };
    let result: any = {};
    try {
      result = await clientAuthHeader.mutate({
        mutation: ADD_POLICY_TAXONOMY,
        variables: {
          policyResourceTaxonAllowInsertInput,
        },
      });
      this.log.info("Added PolicyTaxonomy: ", result);
      return result;
    } catch (err) {
      this.log.error("Error while Add PolicyTaxonomy data in DB", err);
      return;
    }
  }

  public async InsertPolicyTaxonomy(
    policyResourceId: string,
    productTaxonomyId: string,
    value: string
  ) {
    var policyResourceTaxonAllowInsertInput: any = {
      policy_resource_id: policyResourceId,
      product_taxonomy_id: productTaxonomyId,
      value: value,
    };
    let result: any = {};
    try {
      result = await clientAuthHeader.mutate({
        mutation: ADD_POLICY_TAXONOMY,
        variables: {
          policyResourceTaxonAllowInsertInput,
        },
      });
      this.log.info("Added PolicyTaxonomy: ", result);
      return result;
    } catch (err) {
      this.log.error("Error while Add PolicyTaxonomy data in DB", err);
      return;
    }
  }

  public async UpdatePolicyTaxonomy(
    policyResourceId: string,
    productName: string,
    value: string
  ) {
    const productTaxonomyNames = ["customer-location"];
    const productTaxonomyData = await this.getDataFromQuery(
      GET_PRODUCT_TAXONOMY_ID,
      {
        productName,
        productTaxonomyNames,
      }
    );

    let result: any = {};
    try {
      result = await clientAuthHeader.mutate({
        mutation: UPDATE_POLICY_TAXONOMY,
        variables: {
          policyResourceId: policyResourceId,
          productTaxonomyId: productTaxonomyData.product_taxonomy[0].id,
          value: value,
        },
      });
      this.log.info("Updated PolicyTaxonomy: ", result);
      return result;
    } catch (err) {
      this.log.error("Error while Updating PolicyTaxonomy data in DB", err);
      return;
    }
  }

  public async UpdateAllPolicyTaxonomy(
    policyResourceId: string,
    productTaxonomyData: string,
    value: string
  ) {
    let result: any = {};
    try {
      result = await clientAuthHeader.mutate({
        mutation: UPDATE_POLICY_TAXONOMY,
        variables: {
          policyResourceId: policyResourceId,
          productTaxonomyId: productTaxonomyData,
          value: value,
        },
      });
      this.log.info("Updated PolicyTaxonomy: ", result);
      return result;
    } catch (err) {
      this.log.error("Error while Updating PolicyTaxonomy data in DB", err);
      return;
    }
  }

  public async getUsers(appName: string, productName: string, customerId: string) {
    let usersInfo;
    var users: any = [];
    try {
      if (customerId !== "*") {
        this.log.info("Getting Users for Customer " + customerId + " for App " + appName);
        const productTaxonomyNames = ["customer-location"];
        usersInfo = await this.getDataFromQuery(
          GET_APP_TAXON_USERS,
          {
            appName,
            productName,
            productTaxonomyNames
          }
        );

        var roleInfo = await this.getDataFromQuery(
          GET_APP_ALL_USERS,
          {
            appName,
            productName
          }
        ); 

        if (usersInfo.policy_resource_taxon_allow[0]) {
          usersInfo.policy_resource_taxon_allow.forEach(userData => {
            if (userData.value && isArray(userData.value)) {
              var customerFound = userData.value.filter(function (userDataValue) {
                if (userDataValue.customers != null) {
                  return userDataValue.customers.includes(Number(customerId));
                }
              });

              if (customerFound[0]) {
                var roleName;
                var policyResource;
                roleInfo.policy_resource.forEach(resourceElement => {
                  resourceElement.policy.subject.group.users.forEach(userElement => {

                    if (userData.policyResource.policy.subject.user && userElement.user.email_id === userData.policyResource.policy.subject.user.email_id) {
                      policyResource = resourceElement;
                      return;
                    }
                  });
                });

                if (policyResource) {
                  var roleElement = policyResource.taxon_allow.filter(function (taxonElement) {
                    return taxonElement.taxonomy.name === "role";
                  });

                  if (roleElement[0]) {
                    var roleInfoArrary = roleElement[0].value[0];
                    if (roleInfoArrary) {
                      roleName = roleInfoArrary["role-name"][0];
                    }
                  }
                }

                if (userData.policyResource.policy.subject.user) {
                  users.push({
                    "name": userData.policyResource.policy.subject.user.display_name,
                    "emailId": userData.policyResource.policy.subject.user.email_id,
                    "role": roleName
                  });
                }
              }
            }
          });
        }
      } else {
        this.log.info("Getting all Users for App " + appName);
        if (appName == "dc-portal") {
          usersInfo = await this.getDataFromQuery(
            GET_DC_APP_ALL_USERS,
            {
              appName,
              productName
            }
          );
        } else {
          usersInfo = await this.getDataFromQuery(
            GET_APP_ALL_USERS,
            {
              appName,
              productName
            }
          );
        }

        usersInfo.policy_resource.forEach(info => {
          var roleNameElement = info.taxon_allow.filter(function (roleInfo) {
            return roleInfo.taxonomy.name === "role";
          });
          var roleName;
          if (roleNameElement[0]) {
            var roleInfoArrary = roleNameElement[0].value[0];
            if (roleInfoArrary) {
              roleName = roleInfoArrary["role-name"][0];
            }
          }

          info.policy.subject.group.users.forEach(userData => {
            var managerEmailId;
            if (userData.user.policies) {
              if (userData.user.policies.policy.statements[0]) {
                userData.user.policies.policy.statements.forEach(statementData => {
                  if (statementData.taxon_allow[0]) {
                    managerEmailId = statementData.taxon_allow[0].value;
                  }
                });
              }
            }
            users.push({
              "name": userData.user.display_name,
              "emailId": userData.user.email_id,
              "role": roleName,
              "manager_emailId": managerEmailId
            });
          });
        });
      }

      return users;
    } catch (err) {
      this.log.error("Error while getting DC Users", err);
      return;
    }
  }

  public async getExtendedUserDetails(appName: any, productName: any, customerId: string) {
    let usersInfo;
    try {
      if (customerId === "*") {
        try{
          this.log.info("Getting All Users for App");
          usersInfo = await this.getDataFromQuery(
            GET_EXTENDED_USERS,
            {
              appName,
              productName,
            }
          );
        }catch (err){
          this.log.info("graphql err:",err);
        }
      }
      var userDump: any = [];
      var groupUsers: any = [];
      let usersManagerEmailId: any = new Array();
      // Parsing userdump data  
      for await (const data of usersInfo.policy_resource) {
            let name: String;
            let emailId: any;
            let appName: any;
            let managerEmailId: String = "";
            let customers: any = [];
            let region: any = [];
            let country: any = []; 
            let userGroups: String = "";
          if(data.policy.subject.user){
               // if user policy exists, retrive all policy and taxonomy data
              name = data.policy.subject.user.display_name;
              emailId = data.policy.subject.user.email_id;
              appName = data.resource.app.display_name;
              if(data.taxon_allow && data.taxon_allow[0]){
                let isManagerIdExist = false;
                    for await (const taxon of data.taxon_allow) {
                      if(taxon.taxonomy.name === "manager-emailid" && taxon.value){
                        isManagerIdExist = true;
                          managerEmailId =  taxon.value;
                          usersManagerEmailId[emailId] = managerEmailId;
                      }
                      if(taxon.taxonomy.name === "customer-location") {
                          if(taxon.value && typeof taxon.value !== "string" ){
                            if(taxon.value && taxon.value[0]){
                              taxon.value.forEach(item => {
                                if(item.customers && item.customers[0]){
                                  item.customers.forEach(customer => {
                                    customers.push(customer)
                                  });
                                }
                                if(item.location && item.location[0]){
                                    item.location.forEach(location => {
                                      if(location.region && location.region[0]){
                                        location.region.forEach(reg => {
                                          region.push(reg);
                                        });
                                      }
                                      if(location.country && location.country[0]){
                                        location.country.forEach(con => {
                                          country.push(con);
                                        });
                                      }
                                    });
                                } 
                              });
                          }
                          }
                      }
                  }
                  if(!isManagerIdExist) {
                    managerEmailId = usersManagerEmailId[emailId]?usersManagerEmailId[emailId]:"";
                  }

              }
              let user = {
                name,
                emailId,
                managerEmailId,
                appName,
                customers,
                region,
                country,
                userGroups
              }
              userDump.push(user);
          } else if (data.policy.subject.group) {
              // Users present in DC/FO user groups
              if(data.policy.subject.group.users[0]){
                  data.policy.subject.group.users.forEach(item => {
                    if(item && item.user) {
                      name = item.user.display_name;
                      emailId = item.user.email_id;
                      appName = data.resource.app.display_name;
                      let userGroup = data.policy.subject.group.name;   // Get user group name
                      let user = {
                        name,
                        emailId,
                        appName,
                        userGroup
                      }
                      groupUsers.push(user);
                    }
                  })
              }
          }
      }
      // Remove duplicate users | Unique users
      groupUsers = Array.from(new Set(groupUsers.map(s => s.emailId)))
                    .map(key => {
                      return {
                        name: groupUsers.filter(user => user.emailId === key)[0].name,
                        emailId: key,
                        appName: [...new Set(groupUsers.filter(user => user.emailId === key).map(next => next.appName))],
                        userGroups: [...new Set(groupUsers.filter(user => user.emailId === key).map(next => next.userGroup))]  // Aggregate all user group names
                      }
                    })
      // Push group only users into user dump              
      groupUsers.forEach(useritem => {
          const result = userDump.find( ({ emailId }) => emailId === useritem.emailId);
          if(!result){
              userDump.push(useritem);
          }
      })
      // Add group data into userdump
      userDump.forEach(user => {
        user.userGroups = groupUsers.filter(item => item.emailId === user.emailId)[0]?.userGroups;
      });
      // Remove duplicate users | Unique users
      const result = Array.from(new Set(userDump.map(s => s.name)))
                      .map(key => {
                        let userEmailId = userDump.filter(user => user.name === key)[0].emailId;
                        return {
                          name: key,
                          emailId: userDump.filter(user => user.name === key)[0].emailId,
                          managerEmailId: usersManagerEmailId[userEmailId],
                          appName: userDump.filter(user => user.name === key).map(next => next.appName).toString(),
                          customers: userDump.filter(user => user.name === key).map(next => next.customers).toString(),
                          region: userDump.filter(user => user.name === key).map(next => next.region).toString(),
                          country: userDump.filter(user => user.name === key).map(next => next.country).toString(),
                          userGroups: [...new Set(userDump.filter(user => user.name === key).map(next => next.userGroups))].toString()
                        }
                      });

      return result;
    } catch (err) {
      this.log.error("Error while getting User Dump", err);
      return;
    }
  }

  public async createUser(emailId: string, name: string) {
    const tenantId = "ecbb95bf-b9ed-45ee-8cdf-4cc7d1a7a70c";
    this.log.info("User does not exist. Creating new User");
    let createUserResult = await this.addUserMutation(emailId, null, emailId, name, emailId);
    if (createUserResult && createUserResult.insert_user && createUserResult.insert_user.returning.length > 0) {
      let resultOfAssociation = await this.addUserTenantAssociation(createUserResult.insert_user.returning[0].id, tenantId);
      return resultOfAssociation;
    }
  }

  public async insertUserToGroup(emailId: string, appName: string, productName: string, groupName: string): Promise<any> {
    let groupsInfo = await this.getDataFromQuery(GET_USER_GROUPS, {
      emailId,
      appName,
      productName

    });
    if (groupsInfo.policy[0] && groupsInfo.policy[0].subject.group) {
      this.log.info("User already belongs to group " + groupsInfo.policy[0].subject.group + ". Not adding to the role");
      return {
        status: false
      };
    }

    const createdByUserIdResult = await this.getCreatedByUserId("chandru.k-s@hp.com");

    let userAndGroupIdResult: any = await this.getUserAndGroupId(emailId, groupName);
    if (userAndGroupIdResult.status === false) {
      return userAndGroupIdResult;
    }

    if (Array.isArray(userAndGroupIdResult.user) && Array.isArray(userAndGroupIdResult.group) && userAndGroupIdResult.user.length > 0) {
      if (userAndGroupIdResult.group.length > 0) {
        let addUserResult: any = await this.addUserToGroup(userAndGroupIdResult.user[0].id, userAndGroupIdResult.group[0].id, createdByUserIdResult.user[0].id);
        if (addUserResult.status === false) {
          return addUserResult;
        }

        if (addUserResult.insert_user_group && Array.isArray(addUserResult.insert_user_group.returning) && addUserResult.insert_user_group.returning.length > 0) {
          delete addUserResult.insert_user_group.returning[0]["__typename"];
          return addUserResult;
        }
      }
    } else {
      this.log.info("Could not add User to the group");
      return {
        status: false
      };
    }
  }

  public async removeUserFromGroup(emailId: string, appName: string, group: string[], callFromV99 = true): Promise<any> {
    let query = (callFromV99)?FETCH_APP_GROUPS:FETCH_ALL_APP_GROUPS;
    const groupsInfo = await this.getDataFromQuery(query, { appName });
    const checkGroupsExistsResult: any = await this.getDataFromQuery(CHECK_GROUPS_EXISTS, { group });

    if (checkGroupsExistsResult && !(checkGroupsExistsResult.group.length === group.length)) {
      return {
        status: false,
        message: 'Group does not exist. '
      };;
    }
    let isValidAppGroup = true;
    if (groupsInfo && groupsInfo.group && groupsInfo.group.length > 0 ) {
      const appGroups = groupsInfo.group.map(group => group.name)
      group.forEach(group => {
        if(!appGroups.includes(group)) {
          isValidAppGroup = false;
        }
      });
    }
    if ( isValidAppGroup ) {
      if(emailId && group && group.length > 0) {
        try {
          const result: any = await clientAuthHeader.mutate({
            mutation: REMOVE_USER_FROM_GROUP,
            variables: { emailId: [emailId], groupName: group },
          });
          this.log.info("Removed user from groups");
          return {
            status: true,
            result,
            message: 'Removed user from group(s). '
          }
        } catch (err) {
          this.log.info("Error while executing mutation REMOVE_USER_FROM_GROUP " + err);
          return {
            status: false,
            message: 'Could not remove User from group'
          };
        }
      } else {
        this.log.info("User or group does not exist");
        return {
          status: false,
          message: 'User or group not exist'
        };
      }
    } else {
      this.log.info('User not present in the group')
      return {
        status: false,
        message: 'Group is not part of the given App. '
      }
    }
  }

  public async verifyAndAddUserToGroup(userId: string, appName: string, group: string[]): Promise<any> {
    const groupsInfo = await this.getDataFromQuery(FETCH_APP_GROUPS, { appName });

    const checkGroupsExistsResult: any = await this.getDataFromQuery(CHECK_GROUPS_EXISTS, { group });

    let groupIds: any = []
    if (checkGroupsExistsResult && !(checkGroupsExistsResult.group.length === group.length)) {
      return {
        status: false,
        message: 'Group does not exist. '
      };;
    }

    checkGroupsExistsResult.group.map((groupInfo:any) => {
      groupIds.push(groupInfo.id)
    })

    let isValidAppGroup = true;
    if (groupsInfo && groupsInfo.group && groupsInfo.group.length > 0 ) {
      const appGroups = groupsInfo.group.map(group => group.name)
      group.forEach(group => {
        if(!appGroups.includes(group)) {
          isValidAppGroup = false;
        }
      });
    }
    if ( isValidAppGroup ) {
      if(userId && group && group.length > 0) {
        try {
          const createdByUserIdResult = await this.getCreatedByUserId("chandru.k-s@hp.com");
          const result = await this.addUserToGroup(userId, groupIds, createdByUserIdResult.user[0].id);
          this.log.info("Added user from groups");
          return {
            status: true,
            result,
            message: 'Added user from group(s). '
          }
        } catch (err) {
          this.log.info("Error while adding user to the group " + err);
          return {
            status: false,
            message: 'Could not add User from group'
          };
        }
      } else {
        this.log.info("User or group does not exist");
        return {
          status: false,
          message: 'User or group not exist'
        };
      }
    } else {
      this.log.info('User not present in the group')
      return {
        status: false,
        message: 'Group is not part of the given App. '
      }
    }
  }

  public async deletePolicyResource(
    policyResourceIds: string[],
  ): Promise<any> {
    try {
      const result: any = await clientAuthHeader.mutate({
        mutation: DELETE_POLICY_RESOURCE,
        variables: {
          policyResourceIds
        },
      });
      return result.data;
    } catch (err) {
      this.log.info("Error in policyUpdate " + err);
      return undefined;
    }
  }

  public async disablePolicyResource(
    policyResourceIds: string[],
  ): Promise<any> {
    try {
      const result: any = await clientAuthHeader.mutate({
        mutation: DISABLE_POLICY_RESOURCE,
        variables: {
          policyResourceIds
        },
      });
      return result.data;
    } catch (err) {
      this.log.info("Error in disable policy resource " + err);
      return undefined;
    }
  }

  public async enablePolicyResource(
    policyResourceIds: string[],
  ): Promise<any> {
    try {
      const result: any = await clientAuthHeader.mutate({
        mutation: ENABLE_POLICY_RESOURCE,
        variables: {
          policyResourceIds
        },
      });
      return result.data;
    } catch (err) {
      this.log.info("Error in disable policy resource " + err);
      return undefined;
    }
  }

  public async getCreatedByUserId(emailId: string): Promise<any> {
    this.log.info("Inside getCreatedByUserId");
    try {
      let result: ApolloQueryResult<any> = await clientAuthHeader.query({
        query: GET_CREATED_BY_USER_ID,
        variables: { emailId: emailId },
      });
      return result.data;
    } catch (err) {
      this.log.info("Error while getting createdBy User Id " + err);
      return undefined;
    }
  }

  public async appPolicyResourceExists(policyId: string, appName: string[]) {
    try {
      const result: ApolloQueryResult<any> = await clientAuthHeader.query({
        query: FETCH_APP_POLICY_RESOURCE,
        variables: { policyId, appName },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while running checkGroupsExists query " + err);
      return {
        status: false
      };
    }
  }

  public async getPolicyDetails(policyId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_POLICY_DETAILS,
        variables: { policyId },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting policy details " + err)
      return undefined;
    }
  }

  public async getPolicyByResourceId(resourceId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_POLICY_RESOURCE,
        variables: { resourceId },
      });
      return result.data.policy_resource[0].policy_id;
    } catch (err) {
      this.log.error("Error getPolicyByResourceId " + err)
      return undefined;
    }
  }

  public async getProductIdbyPermissionId(permissionId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_PRODUCT_ID_BY_PERMISSION_ID,
        variables: { id:permissionId },
      });
      return result.data.product_permission[0].product_id;
    } catch (err) {
      this.log.error("Error getProductIdbyPermissionId" + err)
      return undefined;
    }
  }

  public async getUserEmailId(updatedBy: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_USER_EMAIL,
        variables: { updatedBy },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting User Email " + err)
      return undefined;
    }
  }

  public async getEntityNameById(entityId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_ENTITY_NAME,
        variables: { entityId },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting entity name by entity Id " + err)
      return undefined;
    }
  }

  public async getEntityNameBySubjectId(subjectId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_ENTITY_NAME_BY_POLICY_SUBJECT,
        variables: { subjectId },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting entity name by policy subject Id  " + err)
      return undefined;
    }
  }

  public async createAuditLog(
    entityType: string,
    entityId: any,
    newData: any,
    updatedBy: string,
    updatedOn:any,
    traceId:string,
    entityName: string
  ): Promise<any> {
    try {
      this.log.info('Inserting audit log: entityType: ' + entityType + 
      ' entityId: ' + entityId + 
      ' newData: ' + newData + 
      ' updatedBy: ' + updatedBy + 
      ' updatedOn: ' + updatedOn + 
      ' traceId: ' +  traceId + 
      ' entityName ' + entityName
      );
      if(!entityName) {
        this.log.info("EntityName missing - Entity delete mutation");
        return undefined;
      }
      const result: any = await clientApiKey.mutate({
        mutation: CREATE_AUDIT_LOG,
        variables: {
          entityType,
          entityId,
          newData,
          updatedBy,
          updatedOn,
          traceId,
          entityName
        },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error in AuditLog " + err);
      return undefined;
    }
  }

  public async getAppDetails(appId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_APP_DETAILS,
        variables: { appId },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting app details " + err)
      return undefined;
    }
  }

  public async getAppAttributesByAppName(appName: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
          query: GET_APP_ATTRIBUTES_BY_APP_NAME,
          variables: { appName },
        });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting app details " + err)
      return undefined;
    }
  }

  public async serviceDiscoveryDocument(productId: string): Promise<any> {
    try {
      let result: ApolloQueryResult<any> = await clientApiKey.query({
        query: SERVICE_DISCOVERY_DOCUMENT,
        variables: { id: productId },
      });
      return result.data;
    } catch (err) {
      this.log.info("Error while getting product information " + err)
      return undefined;
    }
  }
  
  public async getProductDetails(productId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_PRODUCT_DETAILS,
        variables: { productId },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting product details " + err)
      return undefined;
    }
  }

  private async getUserAndGroupId(emailId: string, groupName: string) {
    try {
      let result: ApolloQueryResult<any> = await clientAuthHeader.query({
        query: GET_USER_AND_GROUP_ID,
        variables: { emailId: emailId, groupName: groupName },
      });
      return result.data;
    } catch (err) {
      this.log.info("Error while running GET_USER_AND_GROUP_ID query " + err);
      return {
        status: false
      };
    }
  }

  private async addUserToGroup(userId: string, groupIds: any, createdByUserId: string): Promise<any> {
    let insertUser: any = [];
    if(Array.isArray(groupIds)) {
      groupIds.map((groupId: any) => {
        let insertUserObj = {
          group_id: groupId,
          user_id: userId,
          created_by: createdByUserId
        };
        insertUser.push(insertUserObj)
      })
    }
    else {
       insertUser = {
        group_id: groupIds,
        user_id: userId,
        // created_by: "bedeae58-628c-4e4c-a9e8-d1a98dfc88e8",
        created_by: createdByUserId
      };
    }
    try {
      let result: any = await clientAuthHeader.mutate({
        mutation: ADD_USER_TO_GROUP,
        variables: { insertUser: insertUser },
      });
      return result.data;
    } catch (err) {
      this.log.info("Error while executing mutation ADD_USER_TO_GROUP " + err);
      return {
        status: false
      };
    }
  }

  public async addUserTenantAssociation(
    userId: string,
    tenantId: string
  ): Promise<any> {
    try {
      let result: any = await clientAuthHeader.mutate({
        mutation: ADD_USER_TENANT_ASSOCIATION,
        variables: { userId: userId, tenantId: tenantId },
      });

      return result.data;
    } catch (err) {
      this.log.info("Error in addUserTenantAssociation " + err);
      return undefined;
    }
  }

  public async checkUserExists(emailId: string): Promise<any> {
    try {
      let result: ApolloQueryResult<any> = await clientAuthHeader.query({
        query: CHECK_USER,
        variables: { emailId: emailId },
      });
      return result.data;
    } catch (err) {
      this.log.info("Error while checking User Exists " + err)
      return undefined;
    }
  }


  public async addUserMutation(
    hpIdUserId: string,
    hpIdResponse: any,
    userName: string,
    displayName: string,
    email: string
  ): Promise<any> {
    try {
      let result: any = await clientAuthHeader.mutate({
        mutation: ADD_USER,
        variables: {
          hpIdUserId: hpIdUserId,
          userName: userName,
          displayName: displayName,
          email: email.toLowerCase(),
          hpIdResponse: hpIdResponse,
        },
      });

      return result.data;
    } catch (err) {
      this.log.info("Error in addUserMutation " + err);
      return undefined;
    }
  }

  async getAppSubjectNameDocument(appProductId:any, subjectId:any,isAppProduct:boolean) {
    try {
      let result = await this.getDataFromQueryByApiKey(GET_APP_SUBJECT_NAME_DOCUMENT, {
        appProductId,
        subjectId,
        isAppProduct
        });
        return result;
    }
    catch (err) {
        this.log.info("Error while running GetAppSubjectNameDocument query " + err);
        return {
            status: false
        };
    }
}

public async UpdateRequestedValue(
  reqId: string,
  requested_value: string
) {
  let result: any = {};
  try {
    requested_value = JSON.parse(requested_value);
    result = await clientApiKey.mutate({
      mutation: UPDATE_REQUESTED_VALUE_MUTATION,
      variables: {
        reqId:reqId,
        requested_value:requested_value
      },
    });
    this.log.info("Updated RequestedValue in request table: ", result);
    return result;
  } catch (err) {
    this.log.error("Error while UpdateRequestedValue data in DB", err);
    return;
  }
}

public async autoRejectUpdate(
  contitionValue: any,
  updateValue: any,
  query: any = REQUEST_NOTIFICATION,
) {
  try {
    if(updateValue  && updateValue.notification_data && updateValue.notification_data.count == 1) {
      query = UPDATE_REQUEST;
    }
    const result: any = await clientApiKey.mutate({
      mutation: query,
      variables: { contitionValue, updateValue },
    });
    return result.data;
  } catch (err) {
    this.log.error("Error while running autoRejectUpdate query " + err);
    return {
      status: err
    };
  }
}

//Get the request where notification has sent atleast once
public async getRequestForAutoReject(
) {
  try {
    let result: ApolloQueryResult<any> = await clientApiKey.query({
      query: GET_REQUEST,
      variables: { },
    });
    return result.data;
  } catch (err) {
    this.log.error("Error while running getRequestForAutoReject query " + err);
    return {
      status: err
    };
  }
}

  private async getDataFromQuery(
    query: any,
    variables: any
  ): Promise<any | undefined> {
    try {
      const result: ApolloQueryResult<any> = await clientAuthHeader.query({
        query,
        variables,
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while querying data from DB", err);
      return;
    }
  }

  private async getDataFromQueryByApiKey(
    query: any,
    variables: any
  ): Promise<any | undefined> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query,
        variables,
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while querying data from DB", err);
      return;
    }
  }

  public async getGroupDetails(entityId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_GROUP_DETAILS,
        variables: { entityId },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting group details " + err);
      return undefined;
    }
  }
  public async getPersonaDetails(entityId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_PERSONA_DETAILS,
        variables: { entityId },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting group details " + err);
      return undefined;
    }
  }

  public async getUserDetails(entityId: any): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_USER_DETAILS,
        variables: { entityId },
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting group details " + err);
      return undefined;
    }
  }

  public async updateRequestComments(
    requestId: string,
    requestUpdateInput: string
  ) {
    let result: any = {};
    try {
      result = await clientApiKey.mutate({
        mutation: UPDATE_REQUEST_COMMENTS,
        variables: {
          requestId: requestId,
          requestUpdateInput: requestUpdateInput
        },
      });
      this.log.info("Updated comments in request table: ", result);
      return result;
    } catch (err) {
      this.log.error("Error while  updating request comments data in DB", err);
      return;
    }
  }

  async getUserInfo(
    userId: any
  ) {
    try {
      let result = await this.getDataFromQueryByApiKey(
        GET_USER_EMAIL,
        {
          updatedBy: userId
        }
      );
      return result;
    } catch (err) {
      this.log.info(
        "Error while running GetAppSubjectNameDocument query " + err
      );
      return {
        status: false,
      };
    }
  }

  public async getAllUser(): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_All_USERS,
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting user details " + err);
      return undefined;
    }
  }
  public async getAllAppsForUser(): Promise<any> {
    try {
      const result: ApolloQueryResult<any> = await clientApiKey.query({
        query: GET_APP_LIST_FOR_USERS,
      });
      return result.data;
    } catch (err) {
      this.log.error("Error while getting user details " + err);
      return undefined;
    }
  }
}