const axios = require("axios");
const https = require("https");
import { Logger, LambdaLogger } from "../lib/lambda-logger/index";
import { stringify } from "querystring";
import { Configuration } from "../config";
import { SecretsManager } from "aws-sdk";

// Call to Notification Service to send mail
export class NitinolService {
  static log: Logger = LambdaLogger.getInstance("NitinolService");
  public static async sendEmail(
    templateName: string,
    toAddress: any,
    fromAddress: any,
    ccAddresses: any,
    bccAddresses: any,
    subject: string,
    templateData: any
  ) {
    let emailResponse: any;
    let accessToken = "";
    const configuration = await Configuration();

    let getSecretValuePath: string = process.env.stage + "/paladin-config-service/keys";
    
    let data: any = await getSecretValue(getSecretValuePath, this.log);

    const authPayLoad = {
      client_id: JSON.parse(data)["nitinol_client_id"],
      client_secret: JSON.parse(data)["nitinol_client_secret"],
      grant_type: "client_credentials"
    };

    let emailPayLoad: any;

    if(fromAddress){
      emailPayLoad = {
        emailTemplateName: templateName,
        toAddress,
        fromAddress,
        ccAddresses,
        bccAddresses,
        withPayloadAsAttachment: false,
        subject,
        templateData,
      };
    }
    else{
      emailPayLoad = {
        emailTemplateName: templateName,
        toAddress,
        ccAddresses,
        bccAddresses,
        withPayloadAsAttachment: false,
        subject,
        templateData,
      };
    }

    try {
      this.log.info("Getting access token...");
      const agent = new https.Agent({
        rejectUnauthorized: false,
      });
      process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
      const tokenResponse: any = await axios.post(
        configuration.emailConfig.emailLogin,
        stringify(authPayLoad),
        {
          headers: {
            "Content-Type": "application/x-www-form-urlencoded",
            Host: configuration.emailConfig.auth.host,
          },
        },
        { httpsAgent: agent }
      );
      if (tokenResponse && tokenResponse.data) {
        this.log.info("Start: Sending Email with payload...", emailPayLoad);
        accessToken = tokenResponse.data.access_token;
        try {
          const response = await axios.post(
            configuration.emailConfig.emailService,
            emailPayLoad,
            {
              headers: {
                "Content-Type": "application/json",
                Authorization: "Bearer " + accessToken,
                Host: configuration.emailConfig.auth.host,
              },
            },
            { httpsAgent: agent }
          );
          if (response && response.data) {
            emailResponse = response.data;
            this.log.info("Nitional Service Email Response", emailResponse);
          } else {
            this.log.error("Unable to get response from nitinol email service");
            emailResponse = {
              error: "Unable to get response from nitinol email service",
            };
          }
        } catch (err) {
          this.log.error("Error occurred in sending email", err);
          return { error: "Error occurred in sending email" };
        }
      } else {
        this.log.error("Unable to get JWT token response from nitinol");
        emailResponse = {
          error: "Unable to get JWT token response from nitinol",
        };
      }
    } catch (err) {
      this.log.error("Error in fetching JWT token" + err);
      return { error: "Error in fetching JWT token" };
    }
    return emailResponse;
  }
}

async function getSecretValue(secretKeyName: string, logger): Promise<any> {
  const options: any = {
    endpoint: "https://secretsmanager.us-west-2.amazonaws.com",
    region: "us-west-2",
  };
  const secretsClient = new SecretsManager(options);

  const params = {
    SecretId: secretKeyName,
  };

  try {
    const data = await secretsClient.getSecretValue(params).promise();
    if (data) {
      if (data.SecretString) {
        return data.SecretString;
      } else {
        return data.SecretBinary;
      }
    } else {
      return false;
    }
  } catch(error) {
    logger.error("Failed to Retrieve the secret value: " + error);
    return false;
  }
}
